from typing import Literal

Tier = Literal["S", "A", "B", "C", "D", "E", "T", "Y"]
DamageType = Literal["Physical", "Special"]
Range = Literal["Ranged", "Melee"]
Difficulty = Literal["Novice", "Intermediate", "Expert"]
Role = Literal["All-Rounder", "Attacker", "Defender", "Speedster", "Supporter"]
Stat = Literal["Attack", "Sp. Attack", "Defense", "Sp. Defense", "HP", "Speed"]
